;***************************************************************************
;
; File Name		:'M168Boot.asm"
; Title			:Mega168 Boot and Self Update rutins
; Date			:2006.04.21.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034, Old: +36-30-9541-658
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:Mega168
;
;***************************************************************************
;	D E S C R I P T I O N
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  why
;	----	----------  ---	--------------------------------------------
;	0.01	2006.04.21  VFX	Creation M88
;	0.02	2006.08.14  VFX modify to M168
;
;***************************************************************************
;
;
;
;Hardware
;***************************************************************************
;
;
;**************************************************************************
;* Const Def
;
;
;***************************************************************************
.DSEG
UserFLASH:	.byte 3			;Belso FLASH irashoz kell, Data pointer to FLASH
AppDataPointer:	.byte 1			;pointer az AppDatan beul
Updating:	.byte 1			;FLASH update flag

;*************************************************************************
;*  Text Const
;*
.CSEG


;***********************************************
; Felkeszules a FLASH irasara
;             					;ide majd kell meg az interruptok tiltasa stb...
Upgrade_Init:
		clr	R0
		sts	UserFLASH+0,R0		;Ez a Z-reg
		sts	UserFLASH+1,R0
		sts	AppDataPointer,R0
		inc	R0
		sts	Updating,R0		;Update now
		ret

;************************************************
;  R16 - hany byte-ot kell betolteni a FLASHbe X-regtol kezdve
;
CopyAppData:
		lds	R17,AppDataPointer
		lds	ZL,UserFLASH+0
		lds	ZH,UserFLASH+1		;Z last Page + word Pointer
		mov	R19,R16
CopyAD1:
		ld	R0,X+
		ld	R1,X+

; Z-register is used to address the data in the temporary buffer
;The temporary buffer will auto-erase after a Page Write operation
;or by writing the RWWSRE bit in SPMCSR
;If the EEPROM is written in the middle of an SPM Page Load operation,
;all data loaded will be lost.
		ldi	R16,0b00000001		;AVR_PageLoading
		rcall	Do_spm

		inc	R17
		cpi	R17,PAGESIZE
		brne	CopyAD2
					     	;itt megtelt az AppData lehet FLASHbe tolni

;Erase
; The page address must be written to PCPAGE in the Z-register.
; The CPU is halted during the Page Erase operation.
;
		ldi	R16,0b00000011			;Page Erase comand
		rcall	Do_spm

;Write
;The page address must be written to PCPAGE.
;Other bits in the Z-pointer must be written to zero during this operation.
;The CPU is halted during the Page Write operation.
;
		mov	R20,ZL				;save ZL
		andi	ZL,0b10000000			;Clear PWWORD area in Z
		ldi	R16,0b00000101			;AVR_PageWrite
		rcall	Do_spm
		mov	ZL,R20				;Restore ZL

		ldi	R16, (1<<RWWSRE) | (1<<SPMEN)	;ReEnable RWW Section
		rcall	Do_spm

Return:
		lds	R21,SPMCSR
		sbrc	R21,RWWSB			; If RWWSB is set, the RWW section is not ready yet
		  rjmp	Return
       							;itt kellene visszaellleorizni az uj flash tartalmat!!!
		clr	R17
CopyAD2:
		adiw	ZL,2
		dec	R19
		dec	R19
		brne	CopyAD1	   		;amig van masolando, addig ciklus

		sts	AppDataPointer,R17
		sts	UserFLASH+0,ZL
		sts	UserFLASH+1,ZH		;Z last Page + word Pointer
		ret


;******************************************************
; Do_spm
;
; In:  R16 - SPM Command
;
; Out: -
;
; ALT: R3
Do_spm:
		in	R3,SPMCSR			; check for previous SPM complete
		sbrc	R3,SPMEN
		 rjmp	Do_spm
							; input: spmcsrval determines SPM action
		in	R3,SREG				; disable interrupts if enabled, store status
		cli
Wait_ee:						; check that no EEPROM write access is present
		sbic	EECR,EEPE			;compatibility with old EEWE bit name
		 rjmp	Wait_ee
							; SPM timed sequence
		out	SPMCSR,R16
		spm
		out	SREG,R3				; restore SREG (to enable interrupts if originally enabled)
		ret

