;***************************************************************************
;
; File Name		:'M88_UARTBoot.asm"
; Title			:ATmega168 Flash Upgrade by UART
; Date			:2006.04.23.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034,  Old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:AtMega88/168
;
;***************************************************************************
;	D E S C R I P T I O N
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who		why
;	----	----------  ---		--------------------------------------------
;	0.01	2006.04.23  VFX		Creation
;
;***************************************************************************
; Hardware
;
;
;**************************************************************************
; Const Def
;



;From the data sheet:BAUD = XTAL/(16*(UBRR+1))
;Rearranging you get:UBRR = XTAL/(16*BAUD) - 1
;
;This would work except that any fraction will be truncated.
;For least error we want to round off the result.
;So we add 1/2 as follows: UBRR = XTAL/(16*BAUD) - 1 + 0.5
;Combining the -1 and the +0.5 we can rewrite this as follows:
;
;UBRR = INT(XTAL/(16*BAUD) - 0.5)

.equ	Rx_Buffer_Length = 16
.equ	Tx_Buffer_Length = 4


;***************************************************************************
.DSEG

Rx_Buffer:
Rx_Head:	.byte 1
Rx_Tail:	.byte 1
Rx_Data:	.byte Rx_Buffer_Length


UARTFlg:	.byte 1			;UART Flags


;*************************************************************************
;*  Text Const
;*
.CSEG


;************************************************************************
;** USART0_Init
;** Set up the inital states for the buffers, and make sure the handshake
;** is turned on (Not the Shaddap state), so we can talk.
;**
;**
;************************************************************************
USART0_Init:

		lds	R16,PRR
		andi	R16,255-(1<<PRUSART0)	;USART0 wake up
		sts	PRR,R16

		clr	R16
		sts	Rx_Tail,R16
		sts	Rx_Head,R16
		sts	UARTFlg,R16

		ldi	R16,High(SYSCLK/(16*BaudSpeed0)-1)
		sts	UBRR0H,R16
		ldi	R16,Low(SYSCLK/(16*BaudSpeed0)-1)
		sts	UBRR0L,R16		;BaudRate Gen.

		ldi	R16,0b01000000		;RXCn, TXCn, UDREn, FEn, DORn, UPEn, U2Xn, MPCMn
		sts	UCSR0A,R16

		ldi	R16,0b10011000		;RXCIE,TXCIE,UDRIE,RXEN,TXEN,UCSZn2,RXB8,TXB8
		sts	UCSR0B,R16

		ldi	R16,0b00000110		;, UMSELn, UPMn1, UPMn0, USBSn, UCSZn1, UCSZn0, UCPOLn
		sts	UCSR0C,R16
		ret


;***********************************************
; UART Tx Interrupt rutinok
;**********************************************
USART0_DRE:	

;**********************************************
;* UART TX COMPLET
;**********************************************
USART0_TXC:
		RETI


SendChrW:
		lds	R0,UCSR0A
		bst	R0,TxC0
		brtc	SendChrW		;Wait while TX reg will free

		bld	R0,TxC0			
		sts	UCSR0A,R0		;Clear Tx Complete Flag

		sts	UDR0,R16
		ret


;*******************************************
;* UART RX COMPLET
;*******************************************
USART0_RXC:	push	ZL
		push	ZH
		in	ZL,SREG			; preserve main OS status reg.
		push	ZL
		push	R16
		push	R2

		clr	R2
		ldi	ZL,low(Rx_Data)		; ptr to our rxbuffer... z = Buf[rxtail]
		ldi	ZH,high(Rx_Data)
		lds	R16,Rx_Head
		add	ZL,R16
		adc	ZH,R2

						;itt ki kellene olvasni a statust, a rafutas, frame error stb miatt

		lds	R2,UDR0			; get the incomming char
		st	Z,R2			; Buf[head] = UDR
		inc	R16
		cpi	R16,Rx_Buffer_Length
		brlo	Rs232inx
		clr	R16			; Circle around to the first buffer pos.
Rs232inx:	sts	Rx_Head,R16
        					;Kiszamolja van-e meg hely a bufferben

RS232IntExit:	pop	R2
		pop	R16
		pop	ZL
		out	SREG,ZL			; restore previous status reg
		pop	ZH
		pop	ZL
		reti				; return to normal pgm status.


;************************************************
;* The RS232 RxD FIFO manager			*
;* In: -                                        *
;* Out C=1, akkor R0=RS232 rx-ed char		*
;************************************************
Rs232GetByte:	clr	R0
		lds	R18,Rx_Tail
		lds	R19,Rx_Head
		cp	R19,R18
		breq	Rs232GetBytexit
		ldi	ZL,low(Rx_Data)		; Pointer to the rxbuffer (FIFO)
		ldi	ZH,high(Rx_Data)
		add	ZL,R18
		adc	ZH,R0			; add address offset...  z = &Buf[Tail]
		ld	R0,Z			; R0 = Buf[Tail++]
		inc	R18
		cpi	R18,Rx_Buffer_Length	; test for the end of the data buffer
		brlo	Rs232Gbx
		clr	R18			; Circle around to the first buffer pos.
Rs232Gbx:	sts	Rx_Tail,R18
		sec				;Valid Char
		ret
Rs232GetBytexit:
		clc				; Return 'No' bytes in yet.
		ret

