;***************************************************************************
;
; File Name		:'Main.asm"
; Title			:M168 Boot Loader for EGZ-019
; Date			:2006.08.14.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034 , OLD: +36-30-9541-658
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:ATmega168
;
;***************************************************************************
;	D E S C R I P T I O N
;
; Fuse bits
;	SELFPROGEN = 0
;	
;       BOOTRTS = 0
;	BOOTSZ10 = 01
;
;	RSTDISBL = 1
;	DWEN = 1
;	SPIEN = 0
;	WDTON = 1
;	EESAVE = 1
;	BODLEVEL = 
;	CKDIV8 = 1
;	CKOOUT = 1
;	SUT = 10
;	CKSEL = 0010
;
;	CLKPS = 2
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  	why
;	----	----------  ---		------------------------------------
;	0.01	2006.04.21  VFX		Creation on M88
;	0.02	2006.08.14  VFX		Modify to M168
;                       
;
;***************************************************************************
;
.include "m168def.inc"
.include "asccie.inc"


;***************************************************************************
;* Const Def

.equ	TRUE	=0xFF		;Yes
.equ	FALSE	=0x00		;No

.equ	MAJOR_REV	='1'
.equ	MINOR_REV	='0'
.equ	MINOR_REVB	='0'

;System

				;Oscillator: f=internal, 8.0000 MHz
.equ	CLKDIV = 0		;CLKPS = 3 - div 8
.equ	SYSCLK = 8000000	;System Clock [Hz]


.equ	BaudSpeed0=9600		;speed of UART

.equ	SYSTACK = 100		;100Hz system interrupt

;**************************************************************************
;* Hardware Def.
;
.include "hw.inc"

;***************************************************************************
;**** VARIABLES
.DSEG
.org 0x0100

SCNT1:	 	.BYTE 2		;16 bit Software Timer1 /LED villogas periodusideje
SCNT2:	 	.BYTE 2		;16 bit Software Timer2 /XMODEM hasznalja
SCNT3:		.byte 2		;16 bit SW timer //
SCNT4:		.byte 2		;16 bit SW timer //
SWTmr0:	 	.BYTE 1		;8 bites Software Timer


SysFlag:	.byte 1		;System Flag
				;0 = 1 RTC Battery was low on startup
				;1 = 1 RTC not Found
				;2 = 1 LCD Not present or damaged
KeyValue:	.byte 1


;***************************************************************************
.ESEG
dumyspace:	.dd 0		;compatibility with 8515 errata


;***************************************************************************
;****		I N T E R R U P T S
;****
;***************************************************************************
.CSEG
.ORG 0x1E00	;512 word Boot Sector


	jmp	RESET			; Reset Handler
	jmp	EXT_INT0		; IRQ0 Handler
	jmp	EXT_INT1		; IRQ1 Handler
	jmp	PC_INT0			; Pin change Req0 Handler
	jmp	PC_INT1			; Pin change Req1 Handler
	jmp	PC_INT2			; Pin change Req2 Handler
	jmp	WDT			; Watchdog Time-out Handler
	jmp	TIM2_COMPA		; Timer2 CompareA Handler
	jmp	TIM2_COMPB		; Timer2 CompareB Handler
	jmp	TIM2_OVF		; Timer2 Overflow Handler
	jmp	TIM1_CAPT		; Timer1 Capture Handler
	jmp	TIM1_COMPA		; Timer1 CompareA Handler
	jmp	TIM1_COMPB		; Timer1 CompareB Handler
	jmp	TIM1_OVF		; Timer1 Overflow Handler
	jmp	TIM0_COMPA		; Timer0 CompareA Handler
	jmp	TIM0_COMPB		; Timer0 CompareB Handler
	jmp	TIM0_OVF		; Timer0 Overflow Handler
	jmp	SPI_STC			; SPI Transfer Complete Handler
	jmp	USART0_RXC		; USART0 RX Complete Handler
	jmp	USART0_DRE		; USART0,UDR Empty Handler
	jmp	USART0_TXC		; USART0 TX Complete Handler
	jmp	AADC			; ADC Conversion Complete Handler
	jmp	EE_RDY			; EEPROM Ready Handler
	jmp	ANA_COMP		; Analog Comparator Handler
	jmp	TWI			; Two-wire Serial Interface Interrupt Handler
	jmp	SPM_RDY			; SPM Ready Handler

.include	"isr.asm"
.include	"M168Boot.asm"
.include	"M88_UARTBoot.asm"
.include	"Timer0_m48_CTC.asm"

;************************************************************************
;************************************************************************
;* RESET
;
RESET:		cli				;GLOBAL INTERRUP DIS.

		sbi	SKEEP_DIR,SKEEP		;Self keeper set, power on
		sbi	SKEEP_PORT,SKEEP

		ldi	R16,0x80		;Enable CLK divisor Reg.
		ldi	R17,CLKDIV		;to save power
		sts	CLKPR,R16
		sts	CLKPR,R17

		ldi	R16, high(RAMEND)	;Setup stack
		out	SPH,R16
		ldi	R16, low(RAMEND)
		out	SPL,R16

		ldi	R16,0b000000001		;MCUCR enable to change IVSEL
		out	MCUCR,R16
		ldi	R16,0b000000010		;MCUCR, IVSEL point to BOOTSection
		out	MCUCR,R16		;[1] IVSEL - 1 Int.Vect. beginning of the Boot Loader section
						;	   - 0 Int.Vect. from 0x0002 beginning of user flash
						;[0] IVCE  - 1 to enable change of the IVSEL bit
						;	   - 0 normal operation

		ldi	R16,0x01		;Sleep Enabled, Idle
		out	SMCR,R16
	
		ldi	R16,0xFF
		sts	PRR,R16			;All device shut down


		rcall	USART0_Init
		rcall	Timer0_Init

		rcall	Xmodem_Init
		rcall	Upgrade_Init

		sei

		ldi	R16,CR
		sts	UDR0,R16


;***********************************************************************
;***********************************************************************
;********     M     A      I      N				********
;********							********
;***********************************************************************
Main:
		sleep
		nop


		rcall	Xmodem_Receive

		lds	R16,Updating
		cpi	R16,1
		breq	Main

FLASHOk2:
		cli

		rcall	Timer0_Shutdown		;Timer stop


		ldi	R16,0b00000001		;MCUCR enable to change IVSEL
		out	MCUCR,R16
		ldi	R16,0b00000000		;MCUCR, IVSEL point to UserSection
		out	MCUCR,R16		;Interrupt Table -> 0x0002
	
		clr	R16			;RXCIE,TXCIE,UDRIE,RXEN,TXEN,UCSZn2,RXB8,TXB8
		sts	UCSR0B,R16		;UART Stop

		jmp	0x0000			;Jump to user area, Fantom reset


.include	"Boot_XModem.asm"


;***************************************************************************
;**** VARIABLES
.DSEG

SWstack:	.byte 256	;256 byte azert kellhet stacknek
MaxUsedRAM:	.BYTE 1		;utolso byte jelzo, csak azert hogy ne lepjuk tul az RAMEND byte-ot
